;-----------------------------------------------------------------------
; Bidirectional Form Change for Macross: Scrambled Valkyrie
; based on Overtech Edition hack by advancedpillow & Cheesemeister
; by blizzz, 12.01.2022
;-----------------------------------------------------------------------

hirom
check title "shooting macross     "

; ######################################################################
; Macros & Defines

macro move_data_block_here(source, size)
    !pointer = <source>
    !end = <source>+<size>
    while !pointer < !end
        db read1(!pointer)
        !pointer #= !pointer+1
    endif
endmacro

macro move_data_block(source, target, size)
    pushpc
    org <target>
    %move_data_block_here(<source>, <size>)
    pullpc
endmacro

; Text table
' ' = $80
'A' = $82
'B' = $83
'C' = $84
'D' = $85
'E' = $86
'F' = $87
'G' = $88
'H' = $89
'I' = $8a
'J' = $8b
'K' = $8c
'L' = $8d
'M' = $8e
'N' = $8f
'O' = $90
'P' = $91
'Q' = $92
'R' = $93
'S' = $94
'T' = $95
'U' = $96
'V' = $97
'W' = $98
'X' = $99
'Y' = $9a
'Z' = $9b
'-' = $9c
'0' = $a0
'1' = $a1
'2' = $a2
'3' = $a3
'4' = $a4
'5' = $a5
'6' = $a6
'7' = $a7
'8' = $a8
'9' = $a9


; ######################################################################
; Bidirectional Form Change

org $8085e6         ; set X to form (used later by .add_form_offset)
    jsr load_x_form

org $80fe93
load_x_form:
    lda $021a       ; change direction, 0 = left, 1 = right
    bne .change_right
    ldx $0a20       ; form to transition to (gets set by change_left / change_right functions)
    rts
.change_right:
    ldx $0a1e       ; current form (gets set after transition is done)
    rts


org $8085f6
    jsr next_animation_frame

org $80ffa0
next_animation_frame:   ; adds a constant to the animation frame number, depending on ship form
    tay             ; store A (animation frame) in Y
    lda $021a       ; change direction, 0 = left, 1 = right
    bne .right
    lda #$04        ; \ invert frame order
    sec             ;  |
    sbc $0a88       ; / $0a88 = animation frame
    bra .add_form_offset
.right:
    tya             ; restore A
.add_form_offset:
    clc
    adc $876f, X    ; animation frame + $a3, $a7 or $4c, depending on form
    rts


org $80bee6
    jmp read_button_input
    nop #8

org $80fee0
read_button_input:
    lda $a6         ; pushed button id, A $0080, B $8000, X $0040, Y $4000, L $2000, R $1000
    bit $0218       ; change left button
    beq .change_left_button_not_pushed
    lda #$0000      ; change left button was pushed
    sta $021a       ; change direction, 0 = left, 1 = right
    jmp change_left
.change_left_button_not_pushed:
    bit $0214       ; change right button
    beq .return
    lda #$0001      ; change right button was pushed
    sta $021a       ; change direction, 0 = left, 1 = right
    jmp $be38       ; change right function
.return:
    rts


%move_data_block($80be38, $80fea0, $3e)  ; copy change_right function

org $80fea0         ; modify it to change_left
change_left:
    skip $06       
    bne .return
    skip $1B
    cmp #$0000
    bne .skip
    lda #$0006      ; wrap around from form $00 (fighter) to $04 (battroid)
.skip:
    dec             ; decrement form
    dec
    skip $06
    beq .return
    skip $09
.return:
    rts

org $80be76         ; modify change_right function since we moved the rts from $bee6 ahead
    rts             ; return from change_right function
    nop


; ######################################################################
; RAM init

!jump_check_loc = read3($81864C)
!jump_sram = $FD714C

org $818640
    lda #$0002
    sta $0220       ; starting stage, $02 = stage 1, $04 = stage 2 etc
    stz $0210       ; unknown, always zero. unused?
    ;stz $021a       ; change form related (overtech, not required)
    stz $021c       ; power down from hits (overtech)
    if !jump_check_loc != !jump_sram
        jmp init
    endif

org $81fd40
init:               ; load default settings
    lda #$0001      ; A = $0001
    sta $020a       ; shot button in settings
    sta $0224       ; high score hi
    inc             ; A = $0002
    sta $020e       ; display stereo in options screen
    sta $0208       ; game rank (difficulty), $00 = easy, $02 = normal, $04 = hard
    asl             ; A = $0004
;    cmp $85fc       ; \  check amount of difficulty options
;    bne $3          ;  | adjust difficulty to normal for overtech hack
;    sta $0208       ; /  no longer needed with Overtech 1.1
    sta $0216       ; change back display (overtech)
    inc             ; A = $0005
    sta $020c       ; change form button in settings
    lda #$0010      ; A = $0010
    sta $0214       ; change form button, A $0080, B $8000, X $0040, Y $4000, L $0020, R $0010
    asl             ; A = $0020
    sta $0218       ; change back button (overtech)
    lda #$8000      ; A = $8000
    sta $de         ; set audio to stereo
    sta $0212       ; shot button, A $0080, B $8000, X $0040, Y $4000, L $0020, R $0010
    stz $021e       ; level select
    stz $0222       ; high score lo
    rts


; ######################################################################
; Options menu modification

org $81f037         ; amount of text, text location, it's okay to read a bit too much
    ldx #$00b2      ; amount of data to copy
    lda menu_items_table_new, X

org $81FC00         ; menu items table
menu_items_table_new:
    %move_data_block_here($81F418, $41)                             ; copy beginning of table
    db ' ', 'B', 'A', 'C', 'K', $06, $66, $03, ' ', ' '             ; add new menu line
    db $04, $20, $06, $8E, $03, ' ', ' ', ' ', ' ', ' ', ' ', ' ', 'N', 'E', 'X', 'T', $06, $A6
    %move_data_block_here($81F45B, $55)                             ; copy end of table

org $81F0A3         ; function pointers for options, only when level select active
    jsr (menu_item_function_pointers_levelselect, X)

org $81F0B1         ; function pointers for menu items actions
menu_item_function_pointers:
    dw $f102, $f149, change_back_menu_function, $f181, $f129, $f222, $0000, $0000, $0000, $0000, $0000

org $81FB40         ; function pointers for menu options, only when level select active
menu_item_function_pointers_levelselect:
    dw $f102, $f149, change_back_menu_function, $f181, $f129, $f1f5, $f222

org $81F0D1         ; jump from top to bottom of list (= amount of settings * 2 + 2)
    lda #$000c      ; normal
    skip $02
    lda #$000e      ; stage select active
    skip $0c        ; jump from bottom to top of list (= amount of settings * 2)
    cmp #$000a      ; normal
    skip $02
    cmp #$000c      ; stage select active

org $81F2E8         ; cursor positions when level select active
    lda menu_item_cursor_positions_levelselect, X

org $81F2F1         ; cursor position without level select active
menu_item_cursor_positions:
    dw $1030, $1048, $1060, $1068, $1078, $1090, $0000, $0000, $0000, $0000, $0000

org $81FB50         ; cursor position when level select active
menu_item_cursor_positions_levelselect:
    dw $1030, $1048, $1060, $1068, $1078, $1090, $10a8

org $81F32E         ; display change form buttons, execution returns at $81f33d
    jmp copy_change_form_buttons_to_display_buffer
    nop #12

org $81FD00         ; copy change form buttons to display buffer
copy_change_form_buttons_to_display_buffer:
    ldx $0216       ; change back button display id
    lda $f4c2, X
    and #$00ff
    ora #$8000
    sta $0e66
    ldx $020c       ; change next button display id
    lda $f4c2, X
    and #$00ff
    ora #$8000
    sta $0e7b
    jmp $f33d

org $81F34C         ; change offsets for values in display buffer
    sta $0e8a, Y    ; sound
    skip $10
    sta $0ea2       ; stage select
    skip $19
    sta $0ebd, Y    ; highscore
    skip $10
    sta $0ebd, Y    ; highscore
    skip $08
    sta $0ebd, Y    ; highscore

org $81fb00         ; change back line active
change_back_menu_function:
    jsr $f1b9       ; read pushed button code ($a6), load $00 - $05 to A, $ff if no button pushed
    bmi .fb0a       ; branch if no button pushed
    sta $0216       ; store change back button display id ($00 - $05)
    bra .fb2f
.fb0a:              ; no button pushed
    ldx $0216
    lda $a6         ; pushed button code
    bit #$0200      ; d-pad left
    bne .fb1a
    bit #$0100      ; d-pad right
    bne .fb22
    rts
.fb1a:              ; scroll value to the left
    dex
    bpl .fb2b
    ldx #$0005
    bra .fb2b
.fb22:              ; scroll value to the right
    inx
    cpx #$0006
    bcc .fb2b
    ldx #$0000
.fb2b:              ; store change back button display id ($00 - $05)
    stx $0216
    txa
.fb2f:              ; store change back button code in $0218 
    asl
    tax
    lda $f4c8, X
    sta $0218
    rts


; ######################################################################
; Configlation -> Configuration

org $81F03A
    lda $fbff, X    ; move pointer to options text table ahead by 1 byte

org $81FBFF
    ; move text table one byte ahead, to account for the longer word
    db $04, $20, $06, $D4, $00
    db 'C', 'O', 'N', 'F', 'I', 'G', 'U', 'R', 'A', 'T', 'I', 'O', 'N'

; move pointers to text in ram by 1
org $81F318
    sta $0e40, Y
    skip $10
    sta $0e52
    skip $1e
    sta $0e8b, Y
    skip $10
    sta $0ea3
    skip $19
    sta $0ebe, Y
    skip $10
    sta $0ebe, Y
    skip $08
    sta $0ebe, Y

org $81FD0C
    sta $0e67
    skip $0c
    sta $0e7c


; ######################################################################
; Change game rank names
;
;org $81F4B0
;    db 'N', 'O', 'R', 'M', 'A', 'L'
;    db 'H', 'A', 'R', 'D', ' ', ' '
;    db 'B', 'R', 'U', 'T', 'A', 'L'


; ######################################################################
; Millia -> Milia

org $87B5AA
    dw $1111, $1112, $0000
