;-----------------------------------------------------------------------
; Macross: Scrambled Valkyrie Plus SRAM
; by blizzz, 12.01.2022
;-----------------------------------------------------------------------

hirom
check title "shooting macross     "

org $00FFD6
    db $02          ; cartridge type ROM, RAM, battery
    skip $1
    db $01      	; set sram (2 KB)

org $818640
    lda #$0002
    sta $0220       ; starting stage, $02 = stage 1, $04 = stage 2 etc
    stz $0210       ; unknown, always zero. unused?
    ;stz $021a       ; change form related (overtech, not required)
    stz $021c       ; power down from hits (overtech)
    jmp load_sram

org $81fd40
init:               ; load default settings
    lda #$0001      ; A = $0001
    sta $020a       ; shot button in settings
    sta $0224       ; high score hi
    inc             ; A = $0002
    sta $020e       ; display stereo in options screen
    sta $0208       ; game rank (difficulty), $00 = easy, $02 = normal, $04 = hard
    asl             ; A = $0004
;    cmp $85fc       ; \  check amount of difficulty options
;    bne $3          ;  | adjust difficulty to normal for overtech hack
;    sta $0208       ; /  no longer needed with Overtech 1.1
    sta $0216       ; change back display (overtech)
    inc             ; A = $0005
    sta $020c       ; change form button in settings
    lda #$0010      ; A = $0010
    sta $0214       ; change form button, A $0080, B $8000, X $0040, Y $4000, L $0020, R $0010
    asl             ; A = $0020
    sta $0218       ; change back button (overtech)
    lda #$8000      ; A = $8000
    sta $de         ; set audio to stereo
    sta $0212       ; shot button, A $0080, B $8000, X $0040, Y $4000, L $0020, R $0010
    stz $021e       ; level select
    stz $0222       ; high score lo
    rts

load_sram:          ; load saved settings
    lda $a06000     ; \ check if sram is initialized
    cmp #$1337      ; /
    beq .load_data
    jmp init        ; save isn't initialized, load default settings
.load_data:
    lda $a06002     ; \ game rank
    sta $0208       ; /
    lda $a06004     ; \ 
    sta $0212       ;  | shot button
    lda $a06006     ;  |
    sta $020a       ; /
    lda $a06008     ; \
    sta $0214       ;  | change button
    lda $a0600a     ;  |
    sta $020c       ; /
    lda $a0600c     ; \
    sta $0216       ;  | change back button
    lda $a0600e     ;  |
    sta $0218       ; /
    lda $a06010     ; \
    sta $0222       ;  | highscore lo
    lda $a06012     ;  | highscore hi
    sta $0224       ; /
    lda $a06014     ; \ level select
    sta $021e       ; /
    lda $a06016     ; \
    sta $de         ;  | stereo / mono
    lda $a06018     ;  |
    sta $020e       ; /
    rts

; enable level select
;org $81ECA4
;    bra $0B

; level 7 scrolling speed
; $80 = 1 pixel per frame
!band1 = $0020
!band2 = $fff0
!band3 = $ffe0
!band4 = $ffd0
!band5 = $ffc0
!band6 = $ffb0
!band7 = $ffa0
!band8 = $ff90

org $81D1AF
    dw !band1, !band2, !band3, !band4, !band5, !band6, !band7, !band8

; SRAM layout
; $1337 initialization flag
; game rank
; shot button
; change button
; change back button (overtech)
; highscore lo
; highscore hi
; level select
; audio

;org $81f539        ; palette load on level clear and game over
;    jsl save_sram
;    nop
;    nop

org $80d5c8         ; scene load
    jml write_sram_check
    nop #2
scene_load:
    ; continue scene load

!main_menu = #$811a
!level_load = #$8591 ; also #$8599
!ending = #$832c
org $8FFA00
write_sram_check:
    cpy !main_menu
    beq .write_sram
    cpy !level_load
    beq .write_sram
    cpy !ending
    bne .return
    lda #$0002      ; \ reset level to 1
    sta $0220       ; /
    lda #$ffff      ; \ enable level select
    sta $021e       ; /
.write_sram:
    lda #$1337      ; \ init flag
    sta $a06000     ; /
    lda $0208       ; \ game rank
    sta $a06002     ; /
    lda $0212       ; \ 
    sta $a06004     ;  | shot button
    lda $020a       ;  |
    sta $a06006     ; /
    lda $0214       ; \
    sta $a06008     ;  | change button
    lda $020c       ;  |
    sta $a0600a     ; /
    lda $0216       ; \
    sta $a0600c     ;  | change back button (overtech)
    lda $0218       ;  |
    sta $a0600e     ; /
    lda $0222       ; \
    sta $a06010     ;  | highscore lo
    lda $0224       ;  | highscore hi
    sta $a06012     ; /
    lda $021e       ; \ level select
    sta $a06014     ; /
    lda $de         ; \
    sta $a06016     ;  | stereo / mono
    lda $020e       ;  |
    sta $a06018     ; /
.return:
    lda $0000, Y    ; \ code we overwrote at $80d5c8
    sta $4352       ; /
    jml scene_load
